﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace TechNote73_ImageResource_CS
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // 비트맵 배열의 인덱스를 체크
        private int m_iIndex;
        // 현재 반복 횟수를 체크
        private int m_iRep;
        // 콤보박스에서 설정한 반복 횟수
        private int m_iReptCnt;
        private Form2 frm2;

        private void Form1_Load(object sender, EventArgs e)
        {
            // 현재Form을 MainForm으로 설정
            smartForm1.MainForm = this;

            // 멤버 변수 초기화
            m_iIndex = 0;
            m_iRep = 0;

            // SmartTimer의 Interval = 1
            smartTimer1.Interval = 1;
            smartTimer2.Interval = 1;

            // 콤보박스에 아이템을 추가하고 기본값을 1000으로 설정
            cbRepeatCount.ItemList.AddItem("500");
            cbRepeatCount.ItemList.AddItem("1000");
            cbRepeatCount.ItemList.AddItem("2000");
            cbRepeatCount.SelectItemIndex = 1;

            // MainForm을 출력
            smartForm1.Show(0);
        }

        // Form이 로딩될 때 이미지를 인스턴싱
        private void smartForm1_OnFormLoadCompleting()
        {
            // Form2를 ChildForm으로 추가
            frm2 = new Form2();
            smartForm1.AddChildForm(frm2);

            // 이미지 인스턴싱 시간을 체크
            SmartX.SmartTimer.StartWatch();

            // 사용 이미지 10개
            smartPictureBox1.AddImageList(Properties.Resources._0);
            smartPictureBox1.AddImageList(Properties.Resources._1);
            smartPictureBox1.AddImageList(Properties.Resources._2);
            smartPictureBox1.AddImageList(Properties.Resources._3);
            smartPictureBox1.AddImageList(Properties.Resources._4);
            smartPictureBox1.AddImageList(Properties.Resources._5);
            smartPictureBox1.AddImageList(Properties.Resources._6);
            smartPictureBox1.AddImageList(Properties.Resources._7);
            smartPictureBox1.AddImageList(Properties.Resources._8);
            smartPictureBox1.AddImageList(Properties.Resources._9);
            
            // 시간 측정 종료
            SmartX.SmartTimer.StopWatch();
        }

        // Form 로딩이 완료되었을 때 인스턴싱 경과시간을 표시
        private void smartForm1_OnFormLoadCompleted()
        {
            labInstance.Text = SmartX.SmartTimer.StopWatchElapsedSecond.ToString("0.00초");
        }

        // 체크박스 클릭 시 텍스트 변경
        private void chkUpgrade_Click(object sender, EventArgs e)
        {
            if (chkUpgrade.Checked == false) 
            { 
                chkUpgrade.Text = "개선 전"; 
            }
            else 
            { 
                chkUpgrade.Text = "개선 후"; 
            }
        }

        // 버튼 클릭에 대한 이벤트
        private void SmartButton_Click(object sender, EventArgs e)
        {
            // 클릭한 버튼이 [시간 체크] 이미지를 지정한 횟수만큼 반복하고 시간을 측정합니다. : Push Button
            if (sender == btnTimeCheck)
            {
                // [출력시작] 버튼이 눌려있다면 출력을 정지한다.
                if (btnStartEnd.ButtonStatus == SmartX.SmartButton.BUTSTATUS.DOWN)
                {
                    smartTimer1.Stop();
                }
                btnTimeCheck.Enabled = false;
                // 현재 반복 횟수를 초기화
                m_iRep = 0;
                // 반복 횟수를 콤보박스에서 선택한 값으로 설정
                m_iReptCnt = Convert.ToInt32(cbRepeatCount.Text);
                // 반복 시작
                smartTimer2.Start();
                // 리소스에서 바로 출력시 시간 측정 시작
                SmartX.SmartTimer.StartWatch();
            }
            // 클릭한 버튼이 [출력시작] 시간을 측정하지 않는 이미지 출력을 시작/정지합니다. : Push Button
            else if (sender == btnStartEnd)
            {
                if (btnStartEnd.ButtonStatus == SmartX.SmartButton.BUTSTATUS.UP)
                {
                    smartTimer1.Stop();
                }
                else
                {
                    // 시간을 체크하지 않는 이미지 반복 출력
                    smartTimer1.Start();
                }
            }
            // 클릭한 버튼이 [다음 폼]
            else
            {
                // 자식 폼을 Show
                smartForm1.Show(1);
            }
        }

        private void smartTimer1_Tick(object sender, EventArgs e)
        {
            // 체크박스 체크 여부에 따라 출력 방식을 결정
            ShowImage(chkUpgrade.Checked);
        }

        private void smartTimer2_Tick(object sender, EventArgs e)
        {
            // 리소스에서 불러오는 방식을 먼저 실행
            if (m_iRep < m_iReptCnt) 
            { 
                ShowImage(false); 
            }
            // 설정한 반복 횟수 이후 객체에서 불러오는 방식을 실행
            else 
            { 
                ShowImage(true); 
            }

            // 반복횟수를 증가시키고 설정한 반복횟수가 되면 다음 방식의 시간체크를 시작
            if (++m_iRep == m_iReptCnt)
            {
                // 리소스에서 불러오는 방식의 시간체크를 종료하고 라벨에 시간을 표시
                SmartX.SmartTimer.StopWatch();
                labBeforeTime.Text = SmartX.SmartTimer.StopWatchElapsedSecond.ToString("0.00초");
                // 객체에서 불러오는 방식의 시간체크 시작
                SmartX.SmartTimer.StartWatch();
            }
            // 현재 반복횟수가 설정한 반복횟수의 2배가 될 때 반복을 종료
            else if (m_iRep == (m_iReptCnt * 2))
            {
                // 객체에서 불러오는 방식의 시간체크를 종료하고 라벨에 시간 표시
                SmartX.SmartTimer.StopWatch();
                labAfterTime.Text = SmartX.SmartTimer.StopWatchElapsedSecond.ToString("0.00초");

                // 반복이 완료되어 타이머를 정지하고 반복횟수를 초기화
                smartTimer2.Stop();
                m_iRep = 0;
                btnTimeCheck.Enabled = true;
                btnTimeCheck.ButtonUp();
            }
        }

        // 이미지를 SmartPictureBox에 출력하는 메서드(인자값 : 개선 전, 후 체크)
        private void ShowImage(bool bSelect)
        {
            // 리소스에서 바로 호출하는 방식 [개선 전 방식]
            if (bSelect == false)
            {
                // 현재 인덱스에 맞는 이미지를 리소스에서 바로 출력
                switch (m_iIndex)
                {
                    case 0:
                        smartPictureBox1.Image = Properties.Resources._0;
                        break;
                    case 1:
                        smartPictureBox1.Image = Properties.Resources._1;
                        break;
                    case 2:
                        smartPictureBox1.Image = Properties.Resources._2;
                        break;
                    case 3:
                        smartPictureBox1.Image = Properties.Resources._3;
                        break;
                    case 4:
                        smartPictureBox1.Image = Properties.Resources._4;
                        break;
                    case 5:
                        smartPictureBox1.Image = Properties.Resources._5;
                        break;
                    case 6:
                        smartPictureBox1.Image = Properties.Resources._6;
                        break;
                    case 7:
                        smartPictureBox1.Image = Properties.Resources._7;
                        break;
                    case 8:
                        smartPictureBox1.Image = Properties.Resources._8;
                        break;
                    case 9:
                        smartPictureBox1.Image = Properties.Resources._9;
                        break;
                    default:
                        break;
                }
            }
            // 이미지를 인스턴스화 한 후 호출하는 방식(개선 후 방식)
            else
            {
                smartPictureBox1.DrawNext();
            }

            // 이미지의 인덱스를 증가시키고 마지막 값이 된 경우 초기화
            m_iIndex = (m_iIndex == 9) ? 0 : ++m_iIndex;
        }

        // 출력 Form이 변경될 때 발생하는 이벤트
        private void smartForm1_OnEventFormChanged(int iFormIndex)
        {
            switch (iFormIndex)
            {
                case 0:
                    // 메모리 체크를 위해 메모리 인포메이션 창을 출력 (왼쪽 하단)
                    smartMemory1.MemoryStatusInformationShow(SmartX.SmartMemory.VIEWPOSITION.BOTTOMLEFT);
                    break;
                case 1:
                    // Form2를 출력하는 경우 MemoryInformation을 닫습니다.
                    smartMemory1.MemoryStatusInformationClose();
                    break;
                default:
                    break;
            }
        }
    }
}